
from osv import fields, osv
from tools.translate import _


import re

# TODO: if company fiscal year seq code is 'False', 
# give the user a descent error message 

class account_fiscalyear_sequence(osv.Model):
    _name = "account.fiscalyear"
    _inherit = "account.fiscalyear"

    def create_fy_sequences(self, cr, uid, ids, context=None):
        """ Create new sequence entry for every sequence that contains '%(year)s' or '%(y)s' in the prefix
            the created sequences are linked to the fiscal years "ids"
        """
        # in account.journal code is actually the prefix of the sequence
        # whereas ir.sequence code is a key to lookup global sequences.

        for fy in self.browse(cr, uid, ids, context=context):
            company = self.pool.get('res.company').browse(cr,uid,fy.company_id.id,context)

            if ( not company.seq_fy_code ):
                raise osv.except_osv('Config error!','Please configure a Fiscal year sequence code for company %s' % company.name)

            # search sequences from the same company as the fiscal year and wich have the year in their
            # prefix code.
            seq_searchargs = [('company_id','=',fy.company_id.id),'|',('prefix','ilike','%\%(year)s%'),('prefix','ilike','%\%(y)s%')]
            obj_seq = self.pool.get('ir.sequence')
            seq_ids = obj_seq.search(cr,uid,seq_searchargs,context=context)

            obj_journal = self.pool.get('account.journal')

            for seq in obj_seq.browse(cr,uid,seq_ids,context=context):
                # only for active sequences !!
                if ( not seq.active ):
                    continue

                # The prefix and name should match the journal !!
                journal_ids = obj_journal.search(cr,uid,[('sequence_id','=',seq.id)],context=context)
                if ( journal_ids ):
                    journal = obj_journal.browse(cr,uid,journal_ids,context=context)[0]
                    journal_prefix = "%s-%s-" % (journal.code.upper(),fy.code.upper())
                    name = "%s (%s) %s" % ( journal.name, fy.code, company.name )
                else:
                    # Replace the 'year' subst vars into the fiscal year codes
                    # i prefer "-" instead of "/" TODO: this should not be hardcoded
                    journal_prefix = seq.prefix.replace('%(year)s',fy.code.upper()).replace('%(y)s',fy.code.upper()).replace('/','-')
                    name = "%s (%s) %s" % ( seq.name, fy.code, company.name )

                prefix = "%s%s-" % (
                    journal_prefix,
                    company.seq_fy_code.upper())


                # search if there is already a sequence for this fiscal year
                seq_fy_searchargs = [('sequence_main_id','=',seq.id),('fiscalyear_id','=',fy.id)]
                obj_seq_fy = self.pool.get('account.sequence.fiscalyear')
                seq_fiscal_id = obj_seq_fy.search(cr,uid,seq_fy_searchargs,context=context)
                # if there is an ID found, just skip and go to the next id.
                if seq_fiscal_id:

                    seq_fy = {
                        'name'             : name,
                        'code'             : seq.code,
                        'implementation'   : seq.implementation,
                        'prefix'           : prefix,
                        'suffix'           : seq.suffix,
                        'padding'          : seq.padding,
                        'company_id'       : company.id,
                        'number_increment' : seq.number_increment,
                        }

                    seq_fiscal = obj_seq_fy.browse(cr,uid,seq_fiscal_id,context=context)[0]
                    obj_seq.write(cr,uid,seq_fiscal.sequence_id.id,seq_fy,context=context)
                    obj_seq.log(cr,uid,seq_fiscal.sequence_id.id,_('Sequence %s has been updated') % name) 

                else:

                    seq_fy = {
                        'name'             : name,
                        'code'             : seq.code,
                        'implementation'   : seq.implementation,
                        'prefix'           : prefix,
                        'suffix'           : seq.suffix,
                        'padding'          : seq.padding,
                        'company_id'       : company.id,
                        'number_increment' : seq.number_increment,
                        }

                    seq_fy_id = obj_seq.create(cr,uid,seq_fy,context=context)

                    ir_seq_fy = {
                        'sequence_id'      : seq_fy_id,
                        'sequence_main_id' : seq.id,
                        'fiscalyear_id'    : fy.id,
                        }

                    obj_seq_fy.create(cr,uid,ir_seq_fy,context=context)
                    obj_seq.log(cr,uid,seq_fy_id,_('Sequence %s has been created') % name) 

